"""
Overview:
    Meta information for pycivitai package.
"""

#: Title of this project (should be `pycivitai`).
__TITLE__ = 'pycivitai'

#: Version of this project.
__VERSION__ = '0.2.0'

#: Short description of the project, will be included in ``setup.py``.
__DESCRIPTION__ = 'Python Client and Model Management for CivitAI.'

#: Author of this project.
__AUTHOR__ = 'narugo1992'

#: Email of the authors'.
__AUTHOR_EMAIL__ = 'narugo@126.com'
